"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureProjectIsDefined = exports.isProjectStandalone = exports.getProjectFromWorkspace = exports.getProjectMainConfigFilePath = exports.getProjectMainFilePath = exports.getProjectOutputPath = exports.getAngularVersion = exports.removeProviderFromBootstrapApplication = exports.addProviderToBootstrapApplication = exports.addRouteDeclarationToNgModule = exports.removeProviderFromNgModule = exports.addProviderToNgModule = exports.removeExportFromNgModule = exports.addExportToNgModule = exports.removeImportFromNgModule = exports.addImportToNgModule = exports.removeDeclarationFromNgModule = exports.addDeclarationToNgModule = exports.removeAngularJsonScript = exports.addAngularJsonScript = exports.removeAngularJsonStyle = exports.addAngularJsonStyle = exports.removeAngularJsonAsset = exports.addAngularJsonAsset = exports.isAngularVersion = exports.ensureIsAngularLibrary = exports.ensureIsAngularApplication = exports.ensureIsAngularWorkspace = void 0;
const core_1 = require("@angular/core");
const schematics_1 = require("@angular-devkit/schematics");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const change_1 = require("@schematics/angular/utility/change");
const json_file_1 = require("@schematics/angular/utility/json-file");
const app_config_1 = require("@schematics/angular/utility/standalone/app_config");
const util_1 = require("@schematics/angular/utility/standalone/util");
const workspace_1 = require("@schematics/angular/utility/workspace");
const workspace_models_1 = require("@schematics/angular/utility/workspace-models");
const path_1 = require("path");
const semver_1 = require("semver");
const ast_utils_2 = require("./ast-utils");
const file_1 = require("./file");
// --- RULE(s) ----
/**
 * Ensures that the workspace, where the schematic is currently running on, is actually an
 * Angular workspace or throws an exception otherwise. The test is done by ensuring the
 * existence of an `angular.json` file.
 * @throws {SchematicsException} An exception if an `angular.json` file was not found.
 * @returns {Rule}
 */
const ensureIsAngularWorkspace = () => (tree) => {
    if (!tree.exists('angular.json')) {
        throw new schematics_1.SchematicsException('Unable to locate a workspace file, are you missing an `angular.json` or `.angular.json` file ?.');
    }
};
exports.ensureIsAngularWorkspace = ensureIsAngularWorkspace;
/**
 * Ensures that a project is actually an Angular application or throws an exception otherwise.
 * @throws {SchematicsException} An exception if the project is not an Angular application.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
const ensureIsAngularApplication = (projectName) => async (tree) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    const project = await (0, exports.getProjectFromWorkspace)(tree, projectName);
    if (project.extensions['projectType'] !== workspace_models_1.ProjectType.Application) {
        throw new schematics_1.SchematicsException('Project is not an Angular application.');
    }
};
exports.ensureIsAngularApplication = ensureIsAngularApplication;
/**
 * Ensures that a project is actually an Angular library or throws an exception otherwise.
 * @throws {SchematicsException} An exception if the project is not an Angular library.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
const ensureIsAngularLibrary = (projectName) => async (tree) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    const project = await (0, exports.getProjectFromWorkspace)(tree, projectName);
    if (project.extensions['projectType'] !== workspace_models_1.ProjectType.Library) {
        throw new schematics_1.SchematicsException('Project is not an Angular library.');
    }
};
exports.ensureIsAngularLibrary = ensureIsAngularLibrary;
/**
 * Executes a rule only if the current Angular version installed in the project satisfies a given range.
 * @param {string} range An Angular version range that must be satisfied.
 * @param {Rule} rule The rule to execute.
 * @returns {Rule}
 */
const isAngularVersion = (range, rule) => () => {
    try {
        return ((0, semver_1.satisfies)((0, exports.getAngularVersion)().full, range)) ? rule : (0, schematics_1.noop)();
    }
    catch {
        return (0, schematics_1.noop)();
    }
};
exports.isAngularVersion = isAngularVersion;
/**
 * Adds a new asset to the `build` and `test` sections of the `angular.json` file.
 * @param {JsonObject|string} value The asset to add.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
const addAngularJsonAsset = (value, projectName) => (tree) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    customizeAngularJsonBuildAndTestSection('add', 'assets', tree, value, projectName);
};
exports.addAngularJsonAsset = addAngularJsonAsset;
/**
 * Removes an asset from the `build` and `test` sections of the `angular.json` file.
 * @param {JsonObject|string} value The asset to remove.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
const removeAngularJsonAsset = (value, projectName) => (tree) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    customizeAngularJsonBuildAndTestSection('remove', 'assets', tree, value, projectName);
};
exports.removeAngularJsonAsset = removeAngularJsonAsset;
/**
 * Adds a new style to the `build` and `test` sections of the `angular.json` file.
 * @param {JsonObject|string} value The style to add.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
const addAngularJsonStyle = (value, projectName) => (tree) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    customizeAngularJsonBuildAndTestSection('add', 'styles', tree, value, projectName);
};
exports.addAngularJsonStyle = addAngularJsonStyle;
/**
 * Removes a style from the `build` and `test` sections of the `angular.json` file.
 * @param {JsonObject|string} value The style to remove.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
const removeAngularJsonStyle = (value, projectName) => (tree) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    customizeAngularJsonBuildAndTestSection('remove', 'styles', tree, value, projectName);
};
exports.removeAngularJsonStyle = removeAngularJsonStyle;
/**
 * Adds a new script to the `build` and `test` sections of the `angular.json` file.
 * @param {JsonObject|string} value The script to add.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
const addAngularJsonScript = (value, projectName) => (tree) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    customizeAngularJsonBuildAndTestSection('add', 'scripts', tree, value, projectName);
};
exports.addAngularJsonScript = addAngularJsonScript;
/**
* Removes a script from the `build` and `test` sections of the `angular.json` file.
* @param {JsonObject|string} value The style to remove.
* @param {string} projectName The name of the project to look for.
* @returns {Rule}
*/
const removeAngularJsonScript = (value, projectName) => (tree) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    customizeAngularJsonBuildAndTestSection('remove', 'scripts', tree, value, projectName);
};
exports.removeAngularJsonScript = removeAngularJsonScript;
/**
 * Inserts a declaration (ex. Component, Pipe, Directive) into an NgModule declarations and also imports that declaration.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the declaration to insert.
 * @param {string} importPath The path of the file containing the declaration to insert.
 * @returns {Rule}
 */
const addDeclarationToNgModule = (filePath, classifiedName, importPath) => (tree) => {
    const sourceFile = (0, file_1.getTsSourceFile)(tree, filePath);
    const changes = (0, ast_utils_1.addDeclarationToModule)(sourceFile, filePath, classifiedName, importPath);
    (0, file_1.commitChanges)(tree, filePath, changes);
};
exports.addDeclarationToNgModule = addDeclarationToNgModule;
/**
 * Removes a declaration (ex. Component, Pipe, Directive) from an NgModule declarations.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the declaration to remove.
 * @returns {Rule}
 */
const removeDeclarationFromNgModule = (filePath, classifiedName) => (tree) => {
    const sourceFile = (0, file_1.getTsSourceFile)(tree, filePath);
    const change = (0, ast_utils_2.removeSymbolFromNgModuleMetadata)(sourceFile, filePath, 'declarations', classifiedName);
    (0, file_1.commitChanges)(tree, filePath, [change]);
};
exports.removeDeclarationFromNgModule = removeDeclarationFromNgModule;
/**
 * Inserts an import (ex. NgModule) into an NgModule imports and also imports that import.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the import to insert.
 * @param {string} importPath The path of the file containing the import to insert.
 * @returns {Rule}
 */
const addImportToNgModule = (filePath, classifiedName, importPath) => (tree) => {
    let sourceFile = (0, file_1.getTsSourceFile)(tree, filePath);
    // Fix: manage module import with `forRoot` or `forChild`
    const matches = new RegExp(/(.*)\.(forRoot|forChild)\(/gm).exec(classifiedName);
    if (matches === null || matches === void 0 ? void 0 : matches.length) {
        const realClassifiedName = matches[1].trim();
        // Remove any entry first
        (0, file_1.commitChanges)(tree, filePath, [
            (0, ast_utils_2.removeSymbolFromNgModuleMetadata)(sourceFile, filePath, 'imports', realClassifiedName)
        ]);
        // Refresh source and add import + metadata
        sourceFile = (0, file_1.getTsSourceFile)(tree, filePath);
        (0, file_1.commitChanges)(tree, filePath, [
            (0, ast_utils_1.insertImport)(sourceFile, filePath, realClassifiedName, importPath),
            ...(0, ast_utils_1.addSymbolToNgModuleMetadata)(sourceFile, filePath, 'imports', classifiedName)
        ]);
    }
    else {
        (0, file_1.commitChanges)(tree, filePath, (0, ast_utils_1.addImportToModule)(sourceFile, filePath, classifiedName, importPath));
    }
};
exports.addImportToNgModule = addImportToNgModule;
/**
 * Removes an import (ex. NgModule) from an NgModule imports.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the import to remove.
 * @returns {Rule}
 */
const removeImportFromNgModule = (filePath, classifiedName) => (tree) => {
    const sourceFile = (0, file_1.getTsSourceFile)(tree, filePath);
    const change = (0, ast_utils_2.removeSymbolFromNgModuleMetadata)(sourceFile, filePath, 'imports', classifiedName);
    (0, file_1.commitChanges)(tree, filePath, [change]);
};
exports.removeImportFromNgModule = removeImportFromNgModule;
/**
 * Inserts an export (ex. Component, Pipe, Directive) into an NgModule exports and also imports that export.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the export to insert.
 * @param {string} importPath The path of the file containing the export to insert.
 * @returns {Rule}
 */
const addExportToNgModule = (filePath, classifiedName, importPath) => (tree) => {
    const sourceFile = (0, file_1.getTsSourceFile)(tree, filePath);
    const changes = (0, ast_utils_1.addExportToModule)(sourceFile, filePath, classifiedName, importPath);
    (0, file_1.commitChanges)(tree, filePath, changes);
};
exports.addExportToNgModule = addExportToNgModule;
/**
 * Removes an export (ex. Component, Pipe, Directive) from an NgModule exports.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the export to remove.
 * @returns {Rule}
 */
const removeExportFromNgModule = (filePath, classifiedName) => (tree) => {
    const sourceFile = (0, file_1.getTsSourceFile)(tree, filePath);
    const change = (0, ast_utils_2.removeSymbolFromNgModuleMetadata)(sourceFile, filePath, 'exports', classifiedName);
    (0, file_1.commitChanges)(tree, filePath, [change]);
};
exports.removeExportFromNgModule = removeExportFromNgModule;
/**
 * Inserts a provider (ex. Service) into an NgModule providers and also imports that provider.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the provider to insert.
 * @param {string} importPath The path of the file containing the provider to insert.
 * @returns {Rule}
 */
const addProviderToNgModule = (filePath, classifiedName, importPath) => (tree) => {
    const sourceFile = (0, file_1.getTsSourceFile)(tree, filePath);
    const changes = (0, ast_utils_1.addProviderToModule)(sourceFile, filePath, classifiedName, importPath);
    (0, file_1.commitChanges)(tree, filePath, changes);
};
exports.addProviderToNgModule = addProviderToNgModule;
/**
 * Removes a provider (ex. Service) from an NgModule providers.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the provider to remove.
 * @returns {Rule}
 */
const removeProviderFromNgModule = (filePath, classifiedName) => (tree) => {
    const sourceFile = (0, file_1.getTsSourceFile)(tree, filePath);
    const change = (0, ast_utils_2.removeSymbolFromNgModuleMetadata)(sourceFile, filePath, 'providers', classifiedName);
    (0, file_1.commitChanges)(tree, filePath, [change]);
};
exports.removeProviderFromNgModule = removeProviderFromNgModule;
/**
 * Inserts a route declaration to a router module (i.e. as a RouterModule declaration).
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} routeLiteral The Route object to insert.
 * @returns {Rule}
 */
const addRouteDeclarationToNgModule = (filePath, routeLiteral) => (tree) => {
    const sourceFile = (0, file_1.getTsSourceFile)(tree, filePath);
    const change = (0, ast_utils_1.addRouteDeclarationToModule)(sourceFile, filePath, routeLiteral);
    (0, file_1.commitChanges)(tree, filePath, [change]);
};
exports.addRouteDeclarationToNgModule = addRouteDeclarationToNgModule;
/**
 * Inserts a provider (ex. provideRouter) into a bootstrapApplication's providers and also imports that provider.
 * @param {string} filePath The path of the file containing the bootstrapApplication call to modify.
 * @param {string} providerName The name of the provider to insert.
 * @param {string} importPath The path of the file containing the provider to insert.
 * @param {boolean} [useImportProvidersFrom=false] Whether or not to use the importProvidersFrom api.
 * @param {number} [indent=2] The indentation used during the insertion.
 * @returns {Rule}
 */
const addProviderToBootstrapApplication = (filePath, providerName, importPath, useImportProvidersFrom = false, indent = 2) => (tree) => {
    let realProviderName = providerName;
    // Fix: manage provider import with arguments (ex: provideRouter(ROUTES))
    const matches = new RegExp(/(.*)\(/gm).exec(providerName);
    if (matches === null || matches === void 0 ? void 0 : matches.length) {
        realProviderName = matches[1].trim();
        // Remove any entry first
        (0, ast_utils_2.removeProviderFromStandaloneApplication)(tree, filePath, realProviderName);
    }
    // Add provider
    (0, ast_utils_2.addProviderToStandaloneApplication)(tree, filePath, providerName, useImportProvidersFrom, indent);
    // Manage import(s)
    const bootstrapApplicationCall = (0, util_1.findBootstrapApplicationCall)(tree, filePath);
    const appConfig = (0, app_config_1.findAppConfig)(bootstrapApplicationCall, tree, filePath);
    const filePathToUse = (appConfig) ? appConfig.filePath : filePath;
    const sourceFile = (0, file_1.getTsSourceFile)(tree, filePathToUse);
    (0, file_1.commitChanges)(tree, filePathToUse, [
        (0, ast_utils_1.insertImport)(sourceFile, filePathToUse, realProviderName, importPath),
        (useImportProvidersFrom) ? (0, ast_utils_1.insertImport)(sourceFile, filePathToUse, 'importsProvidersFrom', '@angular/core') : new change_1.NoopChange()
    ]);
};
exports.addProviderToBootstrapApplication = addProviderToBootstrapApplication;
/**
 * Removes a provider (ex. provideRouter) from a bootstrapApplication's providers.
 * @param {string} filePath The path of the file containing the bootstrapApplication call to modify.
 * @param {string} providerName The name of the provider to remove.
 * @returns {Rule}
 */
const removeProviderFromBootstrapApplication = (filePath, providerName) => (tree) => {
    (0, ast_utils_2.removeProviderFromStandaloneApplication)(tree, filePath, providerName);
};
exports.removeProviderFromBootstrapApplication = removeProviderFromBootstrapApplication;
// --- HELPER(s) ---
/**
 * Gets the version of Angular currently used in the project.
 * @returns {Version}
 */
const getAngularVersion = () => core_1.VERSION;
exports.getAngularVersion = getAngularVersion;
/**
 * Gets a project output path as defined in the `angular.json` file.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} projectName The name of the project to look for.
 * @throws {SchematicsException} An exception if a `defaultProject` property was not found in the `angular.json` file.
 * @returns {string} The default project output path.
 */
const getProjectOutputPath = (tree, projectName) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    const angularJson = new json_file_1.JSONFile(tree, 'angular.json');
    return angularJson.get(['projects', projectName, 'architect', 'build', 'options', 'outputPath']);
};
exports.getProjectOutputPath = getProjectOutputPath;
/**
 * Gets a project main file path as defined in the `angular.json` file.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} projectName The name of the project to look for.
 * @returns {string} The default project main file path.
 */
const getProjectMainFilePath = (tree, projectName) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    const angularJson = new json_file_1.JSONFile(tree, 'angular.json');
    const buildOptions = angularJson.get(['projects', projectName, 'architect', 'build']);
    return ((buildOptions === null || buildOptions === void 0 ? void 0 : buildOptions.builder) === workspace_models_1.Builders.Application) ? buildOptions === null || buildOptions === void 0 ? void 0 : buildOptions.options['browser'] : buildOptions === null || buildOptions === void 0 ? void 0 : buildOptions.options['main'];
};
exports.getProjectMainFilePath = getProjectMainFilePath;
/**
 * Gets a standalone project main config file path.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} projectName The name of the project to look for.
 * @returns {string|null} The project main config file path if found or null otherwise.
 */
const getProjectMainConfigFilePath = (tree, projectName) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    const appConfig = (0, ast_utils_2.getStandaloneApplicationConfig)(tree, (0, exports.getProjectMainFilePath)(tree, projectName));
    return appConfig ? appConfig.filePath : null;
};
exports.getProjectMainConfigFilePath = getProjectMainConfigFilePath;
/**
 * Gets a project definition object from the current Angular workspace.
 * @async
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} projectName The name of the project to look for.
 * @throws {SchematicsException} An exception if no project was found.
 * @returns {Promise<LibraryDefinition | ApplicationDefinition>} A project definition object.
 */
const getProjectFromWorkspace = async (tree, projectName) => {
    (0, exports.ensureProjectIsDefined)(projectName);
    const workspace = await (0, workspace_1.getWorkspace)(tree);
    const project = workspace.projects.get(projectName);
    if (!project) {
        throw new schematics_1.SchematicsException(`Project "${projectName}" was not found in the current workspace.`);
    }
    const options = {
        name: projectName,
        ...project,
        pathFromRoot: (path) => { var _a; return (0, path_1.join)((_a = project.root) !== null && _a !== void 0 ? _a : '', path); },
        pathFromSourceRoot: (path) => { var _a; return (0, path_1.join)((_a = project.sourceRoot) !== null && _a !== void 0 ? _a : '', path); }
    };
    if (project.extensions['projectType'] === workspace_models_1.ProjectType.Application) {
        /**
         *  Using tree.getDir().subfiles as tree.exists() does not work for directory only
         */
        let assetsPath = options.pathFromRoot('public');
        if (!tree.getDir(assetsPath).subfiles.length) {
            assetsPath = options.pathFromSourceRoot('assets');
            if (!tree.getDir(assetsPath).subfiles.length) {
                assetsPath = null;
            }
        }
        return {
            ...options,
            isStandalone: (0, exports.isProjectStandalone)(tree, projectName),
            mainFilePath: (0, exports.getProjectMainFilePath)(tree, projectName),
            mainConfigFilePath: (0, exports.getProjectMainConfigFilePath)(tree, projectName),
            outputPath: (0, exports.getProjectOutputPath)(tree, projectName),
            assetsPath
        };
    }
    return options;
};
exports.getProjectFromWorkspace = getProjectFromWorkspace;
/**
 * Checks if a project if of type standalone.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} projectName The name of the project to look for.
 * @returns {boolean} Whether or not the project is of type standalone.
 */
const isProjectStandalone = (tree, projectName) => {
    try {
        const mainFilePath = (0, exports.getProjectMainFilePath)(tree, projectName);
        const bootstrapApplicationCall = (0, util_1.findBootstrapApplicationCall)(tree, mainFilePath);
        return (bootstrapApplicationCall !== null);
    }
    catch {
        return false;
    }
};
exports.isProjectStandalone = isProjectStandalone;
/**
 * @internal
 */
const ensureProjectIsDefined = (projectName) => {
    if (!projectName) {
        throw new schematics_1.SchematicsException('Project cannot be determined and no --project option was provided.');
    }
};
exports.ensureProjectIsDefined = ensureProjectIsDefined;
const customizeAngularJsonBuildAndTestSection = (action, option, tree, value, projectName) => {
    const angularJson = new json_file_1.JSONFile(tree, 'angular.json');
    const architectPath = ['projects', projectName, 'architect'];
    ['build', 'test'].forEach(configName => {
        const path = [...architectPath, configName, 'options', option];
        const values = angularJson.get(path);
        const stringifiedValue = values.map(opt => JSON.stringify(opt));
        if ((action === 'add') && !stringifiedValue.includes(JSON.stringify(value))) {
            values.push(value);
            angularJson.modify(path, values);
        }
        else if (action === 'remove') {
            const valueIndex = stringifiedValue.indexOf(JSON.stringify(value));
            if (valueIndex !== -1) {
                values.splice(valueIndex, 1);
                angularJson.modify(path, values);
            }
        }
    });
};
