import { JsonObject } from '@angular-devkit/core';
import { ProjectDefinition } from '@angular-devkit/core/src/workspace';
import { Chainable, ChainableType } from './chainable';
import { ChainableWorkspaceContext } from './chainable-workspace';
declare class ChainableProject<P extends ApplicationDefinition | LibraryDefinition, C extends ChainableWorkspaceContext> extends Chainable<C> {
    protected chainableType: ChainableType;
    protected projectName: string;
    protected _project?: P;
    constructor(chainableType: ChainableType, projectName: string);
    /**
     * @see {@link addDeclarationToNgModule}
     */
    addDeclarationToNgModule(filePath: string, classifiedName: string, importPath: string): this;
    /**
     * @see {@link removeDeclarationFromNgModule}
     * @returns {this}
     */
    removeDeclarationFromNgModule(filePath: string, classifiedName: string): this;
    /**
     * @see {@link addImportToNgModule}
     * @returns {this}
     */
    addImportToNgModule(filePath: string, classifiedName: string, importPath: string): this;
    /**
     * @see {@link removeImportFromNgModule}
     * @returns {this}
     */
    removeImportFromNgModule(filePath: string, classifiedName: string): this;
    /**
     * @see {@link addExportToNgModule}
     * @returns {this}
     */
    addExportToNgModule(filePath: string, classifiedName: string, importPath: string): this;
    /**
     * @see {@link removeExportFromNgModule}
     * @returns {this}
     */
    removeExportFromNgModule(filePath: string, classifiedName: string): this;
    /**
     * @see {@link addProviderToNgModule}
     * @returns {this}
     */
    addProviderToNgModule(filePath: string, classifiedName: string, importPath: string): this;
    /**
     * @see {@link removeProviderFromNgModule}
     * @returns {this}
     */
    removeProviderFromNgModule(filePath: string, classifiedName: string): this;
    /**
     * @see {@link addRouteDeclarationToNgModule}
     * @returns {this}
     */
    addRouteDeclarationToNgModule(filePath: string, routeLiteral: string): this;
    /**
     * @see {@link addProviderToBootstrapApplication}
     * @returns {this}
     */
    addProviderToBootstrapApplication(filePath: string, providerName: string, importPath: string, useImportProvidersFrom?: boolean, indent?: number): this;
    /**
     * @see {@link removeProviderFromBootstrapApplication}
     * @returns {this}
     */
    removeProviderFromBootstrapApplication(filePath: string, providerName: string): this;
    /**
     * @see {@link addAngularJsonAsset}
     * @returns {this}
     */
    addAngularJsonAsset(value: JsonObject | string): this;
    /**
     * @see {@link removeAngularJsonAsset}
     * @returns {this}
     */
    removeAngularJsonAsset(value: JsonObject | string): this;
    /**
     * @see {@link addAngularJsonStyle}
     * @returns {this}
     */
    addAngularJsonStyle(value: JsonObject | string): this;
    /**
     * @see {@link removeAngularJsonStyle}
     * @returns {this}
     */
    removeAngularJsonStyle(value: JsonObject | string): this;
    /**
     * @see {@link addAngularJsonScript}
     * @returns {this}
     */
    addAngularJsonScript(value: JsonObject | string): this;
    /**
     * @see {@link removeAngularJsonScript}
     * @returns {this}
     */
    removeAngularJsonScript(value: JsonObject | string): this;
    protected getContext(): C;
    protected pathFromRoot(path: string): string;
}
export interface LibraryDefinition extends ProjectDefinition {
    name: string;
    pathFromRoot: (path: string) => string;
    pathFromSourceRoot: (path: string) => string;
}
export interface ApplicationDefinition extends LibraryDefinition {
    isStandalone: boolean;
    mainFilePath: string;
    mainConfigFilePath: string | null;
    outputPath: string;
    assetsPath: string | null;
}
export interface ChainableApplicationContext extends ChainableWorkspaceContext {
    project: ApplicationDefinition;
}
export declare class ChainableApplication extends ChainableProject<ApplicationDefinition, ChainableApplicationContext> {
}
export declare const application: (projectName: string) => ChainableApplication;
export interface ChainableLibraryContext extends ChainableWorkspaceContext {
    project: LibraryDefinition;
}
export declare class ChainableLibrary extends ChainableProject<LibraryDefinition, ChainableLibraryContext> {
}
export declare const library: (projectName: string) => ChainableLibrary;
export {};
