"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.library = exports.ChainableLibrary = exports.application = exports.ChainableApplication = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const workspace_models_1 = require("@schematics/angular/utility/workspace-models");
const path_1 = require("path");
const angular_1 = require("../angular");
const chainable_1 = require("./chainable");
class ChainableProject extends chainable_1.Chainable {
    constructor(chainableType, projectName) {
        super(chainableType);
        this.chainableType = chainableType;
        this.projectName = projectName;
        // Sanity checks
        (0, angular_1.ensureProjectIsDefined)(projectName);
        switch (this.chainableType) {
            case chainable_1.ChainableType.APPLICATION:
                this.addRuleToChain(() => (0, angular_1.ensureIsAngularApplication)(this.projectName));
                break;
            case chainable_1.ChainableType.LIBRARY:
                this.addRuleToChain(() => (0, angular_1.ensureIsAngularLibrary)(this.projectName));
                break;
            default:
                throw new schematics_1.SchematicsException(`Context type "${this.chainableType}" is not supported.`);
        }
        // Make sure the second thing we do in the chain (ie. after the super) is to get a reference to the project
        this.addRuleToChain(() => async (tree) => {
            this._project = await (0, angular_1.getProjectFromWorkspace)(tree, this.projectName);
        });
    }
    /**
     * @see {@link addDeclarationToNgModule}
     */
    addDeclarationToNgModule(filePath, classifiedName, importPath) {
        return this.addRuleToChain(() => (0, angular_1.addDeclarationToNgModule)(this.pathFromRoot(filePath), classifiedName, importPath));
    }
    /**
     * @see {@link removeDeclarationFromNgModule}
     * @returns {this}
     */
    removeDeclarationFromNgModule(filePath, classifiedName) {
        return this.addRuleToChain(() => (0, angular_1.removeDeclarationFromNgModule)(this.pathFromRoot(filePath), classifiedName));
    }
    /**
     * @see {@link addImportToNgModule}
     * @returns {this}
     */
    addImportToNgModule(filePath, classifiedName, importPath) {
        return this.addRuleToChain(() => (0, angular_1.addImportToNgModule)(this.pathFromRoot(filePath), classifiedName, importPath));
    }
    /**
     * @see {@link removeImportFromNgModule}
     * @returns {this}
     */
    removeImportFromNgModule(filePath, classifiedName) {
        return this.addRuleToChain(() => (0, angular_1.removeImportFromNgModule)(this.pathFromRoot(filePath), classifiedName));
    }
    /**
     * @see {@link addExportToNgModule}
     * @returns {this}
     */
    addExportToNgModule(filePath, classifiedName, importPath) {
        return this.addRuleToChain(() => (0, angular_1.addExportToNgModule)(this.pathFromRoot(filePath), classifiedName, importPath));
    }
    /**
     * @see {@link removeExportFromNgModule}
     * @returns {this}
     */
    removeExportFromNgModule(filePath, classifiedName) {
        return this.addRuleToChain(() => (0, angular_1.removeExportFromNgModule)(this.pathFromRoot(filePath), classifiedName));
    }
    /**
     * @see {@link addProviderToNgModule}
     * @returns {this}
     */
    addProviderToNgModule(filePath, classifiedName, importPath) {
        return this.addRuleToChain(() => (0, angular_1.addProviderToNgModule)(this.pathFromRoot(filePath), classifiedName, importPath));
    }
    /**
     * @see {@link removeProviderFromNgModule}
     * @returns {this}
     */
    removeProviderFromNgModule(filePath, classifiedName) {
        return this.addRuleToChain(() => (0, angular_1.removeProviderFromNgModule)(this.pathFromRoot(filePath), classifiedName));
    }
    /**
     * @see {@link addRouteDeclarationToNgModule}
     * @returns {this}
     */
    addRouteDeclarationToNgModule(filePath, routeLiteral) {
        return this.addRuleToChain(() => (0, angular_1.addRouteDeclarationToNgModule)(this.pathFromRoot(filePath), routeLiteral));
    }
    /**
     * @see {@link addProviderToBootstrapApplication}
     * @returns {this}
     */
    addProviderToBootstrapApplication(filePath, providerName, importPath, useImportProvidersFrom = false, indent = 2) {
        return this.addRuleToChain(() => (0, angular_1.addProviderToBootstrapApplication)(this.pathFromRoot(filePath), providerName, importPath, useImportProvidersFrom, indent));
    }
    /**
     * @see {@link removeProviderFromBootstrapApplication}
     * @returns {this}
     */
    removeProviderFromBootstrapApplication(filePath, providerName) {
        return this.addRuleToChain(() => (0, angular_1.removeProviderFromBootstrapApplication)(this.pathFromRoot(filePath), providerName));
    }
    /**
     * @see {@link addAngularJsonAsset}
     * @returns {this}
     */
    addAngularJsonAsset(value) {
        return this.addRuleToChain(() => (0, angular_1.addAngularJsonAsset)(value, this.projectName));
    }
    /**
     * @see {@link removeAngularJsonAsset}
     * @returns {this}
     */
    removeAngularJsonAsset(value) {
        return this.addRuleToChain(() => (0, angular_1.removeAngularJsonAsset)(value, this.projectName));
    }
    /**
     * @see {@link addAngularJsonStyle}
     * @returns {this}
     */
    addAngularJsonStyle(value) {
        return this.addRuleToChain(() => (0, angular_1.addAngularJsonStyle)(value, this.projectName));
    }
    /**
     * @see {@link removeAngularJsonStyle}
     * @returns {this}
     */
    removeAngularJsonStyle(value) {
        return this.addRuleToChain(() => (0, angular_1.removeAngularJsonStyle)(value, this.projectName));
    }
    /**
     * @see {@link addAngularJsonScript}
     * @returns {this}
     */
    addAngularJsonScript(value) {
        return this.addRuleToChain(() => (0, angular_1.addAngularJsonScript)(value, this.projectName));
    }
    /**
     * @see {@link removeAngularJsonScript}
     * @returns {this}
     */
    removeAngularJsonScript(value) {
        return this.addRuleToChain(() => (0, angular_1.removeAngularJsonScript)(value, this.projectName));
    }
    // --- OVERRIDE(s) ---
    getContext() {
        return {
            ...super.getContext(),
            project: this._project
        };
    }
    pathFromRoot(path) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        if (path.startsWith('__SRC__')) {
            return path.replace('__SRC__', (_b = (_a = this._project) === null || _a === void 0 ? void 0 : _a.sourceRoot) !== null && _b !== void 0 ? _b : 'src');
        }
        else if (((_c = this._project) === null || _c === void 0 ? void 0 : _c.extensions['projectType']) === workspace_models_1.ProjectType.Application) {
            const p = this._project;
            if (path.startsWith('__OUTPUT__')) {
                return path.replace('__OUTPUT__', (_d = p.outputPath) !== null && _d !== void 0 ? _d : 'dist');
            }
            else if (path.startsWith('__ASSETS__')) {
                return path.replace('__ASSETS__', (_e = p.assetsPath) !== null && _e !== void 0 ? _e : 'public');
            }
            else if (path === '__MAIN__') {
                return (_f = p.mainFilePath) !== null && _f !== void 0 ? _f : path;
            }
            else if (path === '__CONFIG__') {
                return (_g = p.mainConfigFilePath) !== null && _g !== void 0 ? _g : path;
            }
        }
        return (0, path_1.join)((_j = (_h = this._project) === null || _h === void 0 ? void 0 : _h.root) !== null && _j !== void 0 ? _j : '', path);
    }
}
class ChainableApplication extends ChainableProject {
}
exports.ChainableApplication = ChainableApplication;
const application = (projectName) => new ChainableApplication(chainable_1.ChainableType.APPLICATION, projectName);
exports.application = application;
class ChainableLibrary extends ChainableProject {
}
exports.ChainableLibrary = ChainableLibrary;
const library = (projectName) => new ChainableLibrary(chainable_1.ChainableType.LIBRARY, projectName);
exports.library = library;
