import { JsonObject } from '@angular-devkit/core';
import { WorkspaceDefinition } from '@angular-devkit/core/src/workspace';
import { SchematicContext, TaskId, Tree } from '@angular-devkit/schematics';
import { PackageItem } from '../package-json';
import { Chainable } from './chainable';
export declare class ChainableWorkspace extends Chainable<ChainableWorkspaceContext> {
    /**
     * @see {@link addPackageJsonDependencies}
     * @returns {this}
     */
    addPackageJsonDependencies(deps: (string | PackageItem)[]): this;
    /**
     * @see {@link addPackageJsonDevDependencies}
     * @returns {this}
     */
    addPackageJsonDevDependencies(deps: (string | PackageItem)[]): this;
    /**
     * @see {@link addPackageJsonPeerDependencies}
     * @returns {this}
     */
    addPackageJsonPeerDependencies(deps: (string | PackageItem)[]): this;
    /**
     * @see {@link removePackageJsonDependencies}
     * @returns {this}
     */
    removePackageJsonDependencies(deps: (string | PackageItem)[]): this;
    /**
     * @see {@link removePackageJsonDevDependencies}
     * @returns {this}
     */
    removePackageJsonDevDependencies(deps: (string | PackageItem)[]): this;
    /**
     * @see {@link removePackageJsonPeerDependencies}
     * @returns {this}
     */
    removePackageJsonPeerDependencies(deps: (string | PackageItem)[]): this;
    /**
     * @see {@link packageInstallTask}
     * @returns {this}
     */
    packageInstallTask(callback?: (taskId?: TaskId) => void, force?: boolean): this;
    /**
     * @see {@link addAngularJsonAsset}
     * @returns {this}
     */
    addAngularJsonAsset(value: JsonObject | string, projectName: string): this;
    /**
     * @see {@link removeAngularJsonAsset}
     * @returns {this}
     */
    removeAngularJsonAsset(value: JsonObject | string, projectName: string): this;
    /**
     * @see {@link addAngularJsonStyle}
     * @returns {this}
     */
    addAngularJsonStyle(value: JsonObject | string, projectName: string): this;
    /**
     * @see {@link removeAngularJsonStyle}
     * @returns {this}
     */
    removeAngularJsonStyle(value: JsonObject | string, projectName: string): this;
    /**
     * @see {@link addAngularJsonScript}
     * @returns {this}
     */
    addAngularJsonScript(value: JsonObject | string, projectName: string): this;
    /**
     * @see {@link removeAngularJsonScript}
     * @returns {this}
     */
    removeAngularJsonScript(value: JsonObject | string, projectName: string): this;
}
export interface ChainableWorkspaceContext {
    tree: Tree;
    schematicContext: SchematicContext;
    workspace: WorkspaceDefinition;
}
export declare const workspace: () => ChainableWorkspace;
