"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.workspace = exports.ChainableWorkspace = void 0;
const angular_1 = require("../angular");
const package_json_1 = require("../package-json");
const chainable_1 = require("./chainable");
class ChainableWorkspace extends chainable_1.Chainable {
    /**
     * @see {@link addPackageJsonDependencies}
     * @returns {this}
     */
    addPackageJsonDependencies(deps) {
        return this.addRuleToChain(() => (0, package_json_1.addPackageJsonDependencies)(deps));
    }
    /**
     * @see {@link addPackageJsonDevDependencies}
     * @returns {this}
     */
    addPackageJsonDevDependencies(deps) {
        return this.addRuleToChain(() => (0, package_json_1.addPackageJsonDevDependencies)(deps));
    }
    /**
     * @see {@link addPackageJsonPeerDependencies}
     * @returns {this}
     */
    addPackageJsonPeerDependencies(deps) {
        return this.addRuleToChain(() => (0, package_json_1.addPackageJsonPeerDependencies)(deps));
    }
    /**
     * @see {@link removePackageJsonDependencies}
     * @returns {this}
     */
    removePackageJsonDependencies(deps) {
        return this.addRuleToChain(() => (0, package_json_1.removePackageJsonDependencies)(deps));
    }
    /**
     * @see {@link removePackageJsonDevDependencies}
     * @returns {this}
     */
    removePackageJsonDevDependencies(deps) {
        return this.addRuleToChain(() => (0, package_json_1.removePackageJsonDevDependencies)(deps));
    }
    /**
     * @see {@link removePackageJsonPeerDependencies}
     * @returns {this}
     */
    removePackageJsonPeerDependencies(deps) {
        return this.addRuleToChain(() => (0, package_json_1.removePackageJsonPeerDependencies)(deps));
    }
    /**
     * @see {@link packageInstallTask}
     * @returns {this}
     */
    packageInstallTask(callback, force = false) {
        return this.addRuleToChain(() => (0, package_json_1.packageInstallTask)(callback, force));
    }
    /**
     * @see {@link addAngularJsonAsset}
     * @returns {this}
     */
    addAngularJsonAsset(value, projectName) {
        return this.addRuleToChain(() => (0, angular_1.addAngularJsonAsset)(value, projectName));
    }
    /**
     * @see {@link removeAngularJsonAsset}
     * @returns {this}
     */
    removeAngularJsonAsset(value, projectName) {
        return this.addRuleToChain(() => (0, angular_1.removeAngularJsonAsset)(value, projectName));
    }
    /**
     * @see {@link addAngularJsonStyle}
     * @returns {this}
     */
    addAngularJsonStyle(value, projectName) {
        return this.addRuleToChain(() => (0, angular_1.addAngularJsonStyle)(value, projectName));
    }
    /**
     * @see {@link removeAngularJsonStyle}
     * @returns {this}
     */
    removeAngularJsonStyle(value, projectName) {
        return this.addRuleToChain(() => (0, angular_1.removeAngularJsonStyle)(value, projectName));
    }
    /**
     * @see {@link addAngularJsonScript}
     * @returns {this}
     */
    addAngularJsonScript(value, projectName) {
        return this.addRuleToChain(() => (0, angular_1.addAngularJsonScript)(value, projectName));
    }
    /**
     * @see {@link removeAngularJsonScript}
     * @returns {this}
     */
    removeAngularJsonScript(value, projectName) {
        return this.addRuleToChain(() => (0, angular_1.removeAngularJsonScript)(value, projectName));
    }
}
exports.ChainableWorkspace = ChainableWorkspace;
const workspace = () => new ChainableWorkspace(chainable_1.ChainableType.WORKSPACE);
exports.workspace = workspace;
