import { JsonValue } from '@angular-devkit/core';
import { WorkspaceDefinition } from '@angular-devkit/core/src/workspace';
import { MergeStrategy, Rule, SchematicContext, Tree } from '@angular-devkit/schematics';
import { InsertionIndex, JSONPath } from '@schematics/angular/utility/json-file';
export declare enum ChainableType {
    WORKSPACE = 0,
    APPLICATION = 1,
    LIBRARY = 2
}
export type ChainableRule<T> = (context: T) => Promise<Rule | void> | Rule | void;
export declare class Chainable<T> {
    protected chainableType: ChainableType;
    protected _rules: (() => Rule)[];
    protected _tree?: Tree;
    protected _schematicContext?: SchematicContext;
    protected _workspace?: WorkspaceDefinition;
    constructor(chainableType: ChainableType);
    /**
     * @see {@link deployFiles}
     * @returns {this}
     */
    deployFiles(templateOptions?: {}, source?: string, destination?: string, strategy?: MergeStrategy): this;
    /**
     * @see {@link deleteFiles}
     * @returns {this}
     */
    deleteFiles(files: string[], force?: boolean): this;
    /**
     * @see {@link renameFile}
     * @returns {this}
     */
    renameFile(from: string, to: string): this;
    /**
     * @see {@link createOrUpdateFile}
     * @returns {this}
     */
    createOrUpdateFile(filePath: string, data: unknown): this;
    /**
     * @see {@link downloadFile}
     * @returns {this}
     */
    downloadFile(source: string | URL, destination: string, replace?: boolean): this;
    /**
     * @see {@link replaceInFile}
     * @returns {this}
     */
    replaceInFile(filePath: string, searchValue: string | RegExp, replaceValue: string): this;
    /**
     * @see {@link addImportToFile}
     * @returns {this}
     */
    addImportToFile(filePath: string, symbolName: string, fileName: string, isDefault?: boolean): this;
    /**
     * @see {@link modifyImportInFile}
     * @returns {this}
     */
    modifyImportInFile(filePath: string, symbolName: string, newSymbolName: string | undefined, fileName: string): this;
    /**
     * @see {@link removeImportFromFile}
     * @returns {this}
     */
    removeImportFromFile(filePath: string, symbolName: string, fileName: string): this;
    /**
     * @see {@link modifyJsonFile}
     * @returns {this}
     */
    modifyJsonFile(filePath: string, jsonPath: JSONPath, value: JsonValue | undefined, insertInOrder?: InsertionIndex | false): this;
    /**
     * @see {@link removeFromJsonFile}
     * @returns {this}
     */
    removeFromJsonFile(filePath: string, jsonPath: JSONPath): this;
    /**
     * @see {@link isAngularVersion}
     * @returns {this}
     */
    isAngularVersion(range: string, rule: ChainableRule<T>): this;
    /**
     * @see {@link log}
     * @returns {this}
     */
    log(message: string): this;
    /**
     * @see {@link logInfo}
     * @returns {this}
     */
    logInfo(message: string): this;
    /**
     * @see {@link logWarning}
     * @returns {this}
     */
    logWarning(message: string): this;
    /**
     * @see {@link logError}
     * @returns {this}
     */
    logError(message: string): this;
    /**
     * @see {@link logAction}
     * @returns {this}
     */
    logAction(message: string): this;
    /**
     * @see {@link spawn}
     * @returns {this}
     */
    spawn(command: string, args: string[], showOutput?: boolean): this;
    /**
     * @see {@link runAtEnd}
     * @returns {this}
     */
    runAtEnd(rule: Rule): this;
    /**
     * Runs a custom rule within the chainable context.
     * @param {ChainableRule<T>} rule The custom rule to be run.
     * @returns {this}
     */
    rule(rule: ChainableRule<T>): this;
    /**
     * Transforms the chainable context into an array of rules.
     * @returns {Rule[]}
     */
    toRules(): Rule[];
    /**
     * Transforms the chainable context into a single rule.
     * @returns {Rule}
     */
    toRule(): Rule;
    protected addRuleToChain(rule: () => Rule): this;
    protected getContext(): T;
    protected pathFromRoot(path: string): string;
    protected init(): void;
}
