import { Rule, TaskId } from '@angular-devkit/schematics';
export interface PackageItem {
    name: string;
    version?: string;
}
/**
 * Triggers a package installation task using the user's preferred package manager.
 * By default the installation will only occur if any packages were previously added or removed to/from the `package.json` file.
 * This default behavior can be bypassed by setting the `force` parameter to `true`.
 * @param {(taskId?: TaskId) => void} [callback] A callback function that will be called with the id of the newly created task.
 * @param {boolean} [force=false] Whether the installation should be made only if changes exists or made no matter what.
 * @returns {Rule}
 */
export declare const packageInstallTask: (callback?: (taskId?: TaskId) => void, force?: boolean) => Rule;
/**
 * Adds items to the `dependencies` section of `package.json` file.
 * @param {(string|PackageItem)[]} deps List of items to be added.
 * @returns {Rule}
 */
export declare const addPackageJsonDependencies: (deps: (string | PackageItem)[]) => Rule;
/**
* Adds items to the `devDependencies` section of `package.json` file.
* @param {(string|PackageItem)[]} deps List of items to be added.
* @returns {Rule}
*/
export declare const addPackageJsonDevDependencies: (deps: (string | PackageItem)[]) => Rule;
/**
* Adds items to the `peerDependencies` section of `package.json` file.
* @param {(string|PackageItem)[]} deps List of items to be added.
* @returns {Rule}
*/
export declare const addPackageJsonPeerDependencies: (deps: (string | PackageItem)[]) => Rule;
/**
 * Removes items from the `dependencies` section of `package.json` file.
 * @param {(string|PackageItem)[]} deps List of items to be removed.
 * @returns {Rule}
 */
export declare const removePackageJsonDependencies: (deps: (string | PackageItem)[]) => Rule;
/**
 * Removes items from the `devDependencies` section of `package.json` file.
 * @param {(string|PackageItem)[]} deps List of items to be removed.
 * @returns {Rule}
 */
export declare const removePackageJsonDevDependencies: (deps: (string | PackageItem)[]) => Rule;
/**
 * Removes items from the `peerDependencies` section of `package.json` file.
 * @param {(string|PackageItem)[]} deps List of items to be removed.
 * @returns {Rule}
 */
export declare const removePackageJsonPeerDependencies: (deps: (string | PackageItem)[]) => Rule;
