"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.removePackageJsonPeerDependencies = exports.removePackageJsonDevDependencies = exports.removePackageJsonDependencies = exports.addPackageJsonPeerDependencies = exports.addPackageJsonDevDependencies = exports.addPackageJsonDependencies = exports.packageInstallTask = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const tasks_1 = require("@angular-devkit/schematics/tasks");
const latest_version_1 = __importDefault(require("@badisi/latest-version"));
const json_file_1 = require("@schematics/angular/utility/json-file");
let packageJsonDepsModified = false;
const modifyPackageJson = (tree, path, items, remove = false) => {
    const pkgJson = new json_file_1.JSONFile(tree, 'package.json');
    if (!pkgJson) {
        throw new schematics_1.SchematicsException('Could not find a `package.json` file at the root of your project.');
    }
    items.forEach((item) => {
        if (remove && pkgJson.get([path, item.name])) {
            packageJsonDepsModified = true;
            pkgJson.remove([path, item.name]);
        }
        else if (pkgJson.get([path, item.name]) !== item.version) {
            packageJsonDepsModified = true;
            pkgJson.modify([path, item.name], item.version);
        }
    });
};
const modifyDeps = async (tree, type, deps, remove = false) => {
    const jsonValues = [];
    // eslint-disable-next-line no-loops/no-loops
    for (const dep of deps) {
        if (typeof dep === 'string') {
            if (remove) {
                jsonValues.push({ name: dep });
            }
            else {
                const pkgVersions = await (0, latest_version_1.default)(dep);
                jsonValues.push({ name: dep, version: (pkgVersions.latest) ? `^${pkgVersions.latest}` : 'latest' });
            }
        }
        else {
            jsonValues.push(dep);
        }
    }
    modifyPackageJson(tree, type, jsonValues, remove);
};
// --- INSTALL DEPS ---
/**
 * Triggers a package installation task using the user's preferred package manager.
 * By default the installation will only occur if any packages were previously added or removed to/from the `package.json` file.
 * This default behavior can be bypassed by setting the `force` parameter to `true`.
 * @param {(taskId?: TaskId) => void} [callback] A callback function that will be called with the id of the newly created task.
 * @param {boolean} [force=false] Whether the installation should be made only if changes exists or made no matter what.
 * @returns {Rule}
 */
const packageInstallTask = (callback, force = false) => (_tree, context) => {
    let taskId;
    if (packageJsonDepsModified || force) {
        packageJsonDepsModified = false;
        taskId = context.addTask(new tasks_1.NodePackageInstallTask());
    }
    if (callback) {
        callback(taskId);
    }
};
exports.packageInstallTask = packageInstallTask;
// --- ADD DEPS ---
/**
 * Adds items to the `dependencies` section of `package.json` file.
 * @param {(string|PackageItem)[]} deps List of items to be added.
 * @returns {Rule}
 */
const addPackageJsonDependencies = (deps) => (tree) => modifyDeps(tree, 'dependencies', deps);
exports.addPackageJsonDependencies = addPackageJsonDependencies;
/**
* Adds items to the `devDependencies` section of `package.json` file.
* @param {(string|PackageItem)[]} deps List of items to be added.
* @returns {Rule}
*/
const addPackageJsonDevDependencies = (deps) => (tree) => modifyDeps(tree, 'devDependencies', deps);
exports.addPackageJsonDevDependencies = addPackageJsonDevDependencies;
/**
* Adds items to the `peerDependencies` section of `package.json` file.
* @param {(string|PackageItem)[]} deps List of items to be added.
* @returns {Rule}
*/
const addPackageJsonPeerDependencies = (deps) => (tree) => modifyDeps(tree, 'peerDependencies', deps);
exports.addPackageJsonPeerDependencies = addPackageJsonPeerDependencies;
// --- REMOVE DEPS ---
/**
 * Removes items from the `dependencies` section of `package.json` file.
 * @param {(string|PackageItem)[]} deps List of items to be removed.
 * @returns {Rule}
 */
const removePackageJsonDependencies = (deps) => (tree) => modifyDeps(tree, 'dependencies', deps, true);
exports.removePackageJsonDependencies = removePackageJsonDependencies;
/**
 * Removes items from the `devDependencies` section of `package.json` file.
 * @param {(string|PackageItem)[]} deps List of items to be removed.
 * @returns {Rule}
 */
const removePackageJsonDevDependencies = (deps) => (tree) => modifyDeps(tree, 'devDependencies', deps, true);
exports.removePackageJsonDevDependencies = removePackageJsonDevDependencies;
/**
 * Removes items from the `peerDependencies` section of `package.json` file.
 * @param {(string|PackageItem)[]} deps List of items to be removed.
 * @returns {Rule}
 */
const removePackageJsonPeerDependencies = (deps) => (tree) => modifyDeps(tree, 'peerDependencies', deps, true);
exports.removePackageJsonPeerDependencies = removePackageJsonPeerDependencies;
