import { JsonObject } from '@angular-devkit/core';
/**
 * Returns the response data of a given url as a buffer object.
 * @async
 * @param {string|URL} url The url where to get the data from.
 * @param {number} [retries=3] The number of times to retry the request in case of failure.
 * @param {number} [backoff=300] The delay (in milliseconds) between retries.
 * @returns {Promise<Buffer>} A `Buffer` object.
 */
export declare const getDataFromUrl: (url: string | URL, retries?: number, backoff?: number) => Promise<Buffer>;
/**
 * Returns the response data of a given url as a JSON object.
 * @async
 * @param {string|URL} url The url where to get the data from.
 * @param {number} [retries=3] The number of times to retry the request in case of failure.
 * @param {number} [backoff=300] The delay (in milliseconds) between retries.
 * @returns {Promise<JsonObject>} A `JsonObject` object.
 */
export declare const getJsonFromUrl: (url: string | URL, retries?: number, backoff?: number) => Promise<JsonObject>;
