import { Rule } from '@angular-devkit/schematics';
/**
 * Outputs a message to the console.
 * By default, the Angular schematic's logger will misplace messages with breaking indentations.
 * This method makes sure that messages are always displayed at the beginning of the current console line.
 * @param {string} message Message to display.
 * @returns {Rule}
 */
export declare const log: (message: string) => Rule;
/**
 * Executes a set of rules by outputing first the name of the associated schematic to the console.
 * The schematic name will be prefixed by the word "SCHEMATIC" printed in magenta and given options can
 * follow inlined, stringified and printed in gray if verbose mode is activated.
 * @param {string} name Name of the schematic to display.
 * @param {Rule[]} rules Set of rules to execute.
 * @param {unknown} [options] Schematic's options to display.
 * @returns {Rule}
 */
export declare const schematic: (name: string, rules: Rule[], options?: unknown) => Rule;
/**
 * Outputs a message to the console, prefixed by the word "INFO" printed in blue.
 * @param {string} message Message to display.
 * @returns {Rule}
 */
export declare const logInfo: (message: string) => Rule;
/**
 * Outputs a message to the console, prefixed by the word "WARNING" printed in yellow.
 * @param {string} message Message to display.
 * @returns {Rule}
 */
export declare const logWarning: (message: string) => Rule;
/**
 * Outputs a message to the console, prefixed by the word "ERROR" printed in red.
 * @param {string} message Message to display.
 * @returns {Rule}
 */
export declare const logError: (message: string) => Rule;
/**
 * Outputs a message to the console, prefixed by the word "ACTION" printed in yellow.
 * @param {string} message Message to display.
 * @returns {Rule}
 */
export declare const logAction: (message: string) => Rule;
/**
 * Spawns a new process using the given command and arguments.
 * By default, the output will not be redirected to the console unless otherwise specified by the `showOutput`
 * parameter or the `--verbose` current schematic process argument.
 * When the output is not redirected to the console, an animated spinner will be displayed to the console to
 * indicates the current process activity, as well as the command and its options displayed inlined and printed
 * in cyan.
 * @param {string} command The command to run.
 * @param {string[]} args List of string arguments to apply to the command.
 * @param {boolean} [showOutput=false] Whether the process output should be redirected to the console or not.
 * @returns {Rule}
 */
export declare const spawn: (command: string, args: string[], showOutput?: boolean) => Rule;
/**
 * Executes a rule at the very end of the schematic.
 * Beware that most of the other helper rules won't work here (especially those that manipulate the tree).
 * Because, at that time, the Angular schematic has already finished running.
 * @param {Rule} rule The rule to execute.
 * @returns {Rule}
 */
export declare const runAtEnd: (rule: Rule, taskName?: string) => Rule;
/**
 * Executes a rule.
 * @param {Rule} rule The rule to execute.
 * @returns {Rule}
 */
export declare const rule: (rule: Rule) => Rule;
