"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.rule = exports.runAtEnd = exports.spawn = exports.logAction = exports.logError = exports.logWarning = exports.logInfo = exports.schematic = exports.log = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const safe_1 = require("@colors/colors/safe");
const child_process_1 = require("child_process");
/**
 * Outputs a message to the console.
 * By default, the Angular schematic's logger will misplace messages with breaking indentations.
 * This method makes sure that messages are always displayed at the beginning of the current console line.
 * @param {string} message Message to display.
 * @returns {Rule}
 */
const log = (message) => (_tree, context) => () => {
    // Avoid indentation by placing the cursor back at the beginning of the current line
    context.logger.info(`\r${message}`);
};
exports.log = log;
/**
 * Executes a set of rules by outputing first the name of the associated schematic to the console.
 * The schematic name will be prefixed by the word "SCHEMATIC" printed in magenta and given options can
 * follow inlined, stringified and printed in gray if verbose mode is activated.
 * @param {string} name Name of the schematic to display.
 * @param {Rule[]} rules Set of rules to execute.
 * @param {unknown} [options] Schematic's options to display.
 * @returns {Rule}
 */
const schematic = (name, rules, options) => {
    const opts = process.argv.includes('--verbose') ? JSON.stringify(options) : undefined;
    return (0, schematics_1.chain)([
        (0, exports.log)(''),
        (0, exports.log)(`${(0, safe_1.magenta)(`${(0, safe_1.black)((0, safe_1.bgMagenta)(' SCHEMATIC '))} 🚀 ${(0, safe_1.white)('[')} ${(0, safe_1.magenta)(name)}${(opts) ? (0, safe_1.gray)(`, ${opts}`) : ''} ${(0, safe_1.white)(']')}`)}`),
        (0, exports.log)(''),
        ...rules,
        (0, exports.runAtEnd)((0, schematics_1.chain)([(0, exports.log)(''), (0, exports.log)(`${(0, safe_1.green)('>')} ${(0, safe_1.black)((0, safe_1.bgGreen)(' DONE '))}\n`)]), '__task_done__')
    ]);
};
exports.schematic = schematic;
/**
 * Outputs a message to the console, prefixed by the word "INFO" printed in blue.
 * @param {string} message Message to display.
 * @returns {Rule}
 */
const logInfo = (message) => (0, exports.log)(`${(0, safe_1.blue)('>')} ${(0, safe_1.black)((0, safe_1.bgBlue)(' INFO '))} ${(0, safe_1.blue)(message)}`);
exports.logInfo = logInfo;
/**
 * Outputs a message to the console, prefixed by the word "WARNING" printed in yellow.
 * @param {string} message Message to display.
 * @returns {Rule}
 */
const logWarning = (message) => (0, exports.log)(`${(0, safe_1.yellow)('>')} ${(0, safe_1.black)((0, safe_1.bgYellow)(' WARNING '))} ${(0, safe_1.yellow)(message)}`);
exports.logWarning = logWarning;
/**
 * Outputs a message to the console, prefixed by the word "ERROR" printed in red.
 * @param {string} message Message to display.
 * @returns {Rule}
 */
const logError = (message) => (0, exports.log)(`${(0, safe_1.red)('>')} ${(0, safe_1.black)((0, safe_1.bgRed)(' ERROR '))} ${(0, safe_1.red)(message)}`);
exports.logError = logError;
/**
 * Outputs a message to the console, prefixed by the word "ACTION" printed in yellow.
 * @param {string} message Message to display.
 * @returns {Rule}
 */
const logAction = (message) => (0, exports.log)(`${(0, safe_1.yellow)('>')} ${(0, safe_1.black)((0, safe_1.bgYellow)(' ACTION '))} ${(0, safe_1.yellow)(message)}`);
exports.logAction = logAction;
/**
 * Spawns a new process using the given command and arguments.
 * By default, the output will not be redirected to the console unless otherwise specified by the `showOutput`
 * parameter or the `--verbose` current schematic process argument.
 * When the output is not redirected to the console, an animated spinner will be displayed to the console to
 * indicates the current process activity, as well as the command and its options displayed inlined and printed
 * in cyan.
 * @param {string} command The command to run.
 * @param {string[]} args List of string arguments to apply to the command.
 * @param {boolean} [showOutput=false] Whether the process output should be redirected to the console or not.
 * @returns {Rule}
 */
const spawn = (command, args, showOutput = false) => async () => {
    const ora = (await Promise.resolve().then(() => __importStar(require('ora')))).default;
    return new Promise((resolve, reject) => {
        var _a, _b;
        const bufferedOutput = [];
        const verbose = showOutput || process.argv.includes('--verbose');
        const cmdText = `${command} ${args.join(' ')}`;
        const spinner = ora({ text: (0, safe_1.cyan)(cmdText) });
        if (!verbose) {
            spinner.start();
        }
        const childProcess = (0, child_process_1.spawn)(command, args, {
            stdio: (verbose) ? 'inherit' : 'pipe',
            shell: true
        });
        childProcess.once('disconnect', resolve);
        childProcess.once('error', error => reject(error));
        childProcess.on('close', (code) => {
            if (code === 0) {
                if (!verbose) {
                    spinner.succeed((0, safe_1.cyan)(cmdText));
                    spinner.stop();
                }
                return resolve();
            }
            else {
                if (!verbose) {
                    spinner.fail((0, safe_1.red)(`${cmdText}\n`));
                    bufferedOutput.forEach(({ stream, data }) => stream.write(data));
                }
                return reject(new schematics_1.UnsuccessfulWorkflowExecution());
            }
        });
        if (!verbose) {
            (_a = childProcess.stdout) === null || _a === void 0 ? void 0 : _a.on('data', (data) => bufferedOutput.push({ stream: process.stdout, data: data }));
            (_b = childProcess.stderr) === null || _b === void 0 ? void 0 : _b.on('data', (data) => bufferedOutput.push({ stream: process.stderr, data: data }));
        }
    });
};
exports.spawn = spawn;
/**
 * Executes a rule at the very end of the schematic.
 * Beware that most of the other helper rules won't work here (especially those that manipulate the tree).
 * Because, at that time, the Angular schematic has already finished running.
 * @param {Rule} rule The rule to execute.
 * @returns {Rule}
 */
const runAtEnd = (rule, taskName) => (tree, context) => {
    var _a;
    const _context = context;
    (_a = _context._scheduledTasks) !== null && _a !== void 0 ? _a : (_context._scheduledTasks = {});
    // Register the task
    const name = taskName !== null && taskName !== void 0 ? taskName : `__task_${Object.keys(_context._scheduledTasks).length}__`;
    if (!_context._scheduledTasks[name]) {
        _context.engine._host.registerTaskExecutor({
            // @ts-expect-error: `callRule` returns Observable<Tree> where Observable<void> is expected, but that's acceptable
            name, create: async () => Promise.resolve(() => (0, schematics_1.callRule)(rule, tree, context))
        });
        // Schedule the task
        _context._scheduledTasks[name] = _context.addTask({ toConfiguration: () => ({ name }) }, Object.values(_context._scheduledTasks));
    }
};
exports.runAtEnd = runAtEnd;
/**
 * Executes a rule.
 * @param {Rule} rule The rule to execute.
 * @returns {Rule}
 */
const rule = (rule) => rule; // eslint-disable-line @typescript-eslint/no-shadow
exports.rule = rule;
