import { Option } from '@angular/cli/src/command-builder/utilities/json-schema';
import { JsonObject } from '@angular-devkit/core';
import { SchematicContext } from '@angular-devkit/schematics';
export interface NgCliOption extends Option {
    hint: string;
}
/**
 * Returns all the options of a specific local or external schematic's schema.
 * @async
 * @param {SchematicContext} context The current schematic context.
 * @param {string} [schematicName="ng-add"] Name of the schematic schema (ex: "drag-drop").
 * @param {string} [packageName="current-schematic-collection-name"] Name of the package containing the schematic schema (ex: "@angular/cli").
 * @param {boolean} [external=false] Whether the schematic is local or external.
 * @param {number} [retries=3] The number of times to retry the request in case of failure.
 * @param {number} [backoff=300] The delay (in milliseconds) between retries.
 * @returns {Promise<NgCliOption[]>} A collection of `NgCliOption` objects.
 */
export declare const getSchematicSchemaOptions: (context: SchematicContext, schematicName?: string, packageName?: string, external?: boolean, retries?: number, backoff?: number) => Promise<NgCliOption[]>;
/**
 * Returns all the default options of a specific local or external schematic's schema.
 * @async
 * @param {SchematicContext} context The current schematic context.
 * @param {string} [schematicName="ng-add"] Name of the schematic schema (ex: "drag-drop").
 * @param {string} [packageName="current-schematic-collection-name"] Name of the package containing the schematic schema (ex: "@angular/cli").
 * @param {boolean} [external=false] Whether the schematic is local or external.
 * @returns {Promise<JsonObject>} A JsonObject containing all the default options.
 */
export declare const getSchematicSchemaDefaultOptions: (context: SchematicContext, schematicName?: string, packageName?: string, external?: boolean) => Promise<JsonObject>;
